// inbuffer.hh
// File input buffer class
// Kevin Puckett Albrecht
// Rev 4
// 19 March 2000 - 27 Oct 2000

#ifndef INBUFFER_HH
#define INBUFFER_HH

/////////////////////////
// DEFINES //////////////
/////////////////////////

#define INBUFFER_LINE_LENGTH 256
#define INBUFFER_EOF 0x7F

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <fstream.h>
#include <string.h>

// Local //
#include "k_string.hh"
#include "error.hh"
#include "common.hh"

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

class InBuffer
{
  public:
    InBuffer (char *asFileName);
    ~InBuffer ();

    char Get ();
    void Seek ();
    char PushBack ();

  private:
    fstream mFile; // input file
    char *msFileName; // filename
    char mText[INBUFFER_LINE_LENGTH]; // text buffer
    int mPos; // current position in mText
    char mCurr; // current character

    char PopLine ();
};

//// Constructor ////
InBuffer::InBuffer (char *asFileName)
{
  msFileName = new char[strlen (asFileName) + 1];
  k_copy (msFileName, asFileName);

  mFile.open (msFileName, ios::in | ios::nocreate);
  if (!mFile.good())
  {
    gpErrorP->Show (EFF_TERMINATE, EP_FILE_OPEN_FAILED, msFileName);
  }

  mCurr = PopLine ();
}

//// Destructor ////
InBuffer::~InBuffer ()
{
  mFile.close ();
  k_del (msFileName);
}

//// Get ////
//// Returns the current character.
///////////////////////////////////
char InBuffer::Get ()
{
  return mCurr;
}

//// Seek ////
//// Sets the current character to the next character.
//////////////////////////////////////////////////////
void InBuffer::Seek ()
{
  if (mCurr == INBUFFER_EOF) // currently at end of file
  {
    return;
  }
  else if (mCurr == 0) // currently at end of line
  {
    mCurr = PopLine ();   
  }
  else
  {
    mCurr = mText[++mPos];
  }
}

//// PushBack ////
//// Puts the last char back in the buffer.
///////////////////////////////////////////
char InBuffer::PushBack ()
{
  mCurr = mText[--mPos];
  return mCurr;
}

//// PopLine ////
//// Put the next line from the file into the buffer.
//// Then return the line's first character.
/////////////////////////////////////////////////////
char InBuffer::PopLine ()
{
  mPos = 0;

  if (mFile.eof())
  {
    return INBUFFER_EOF;
  }
  else
  {
    mFile.getline (mText, INBUFFER_LINE_LENGTH);
    gpCommon->nCurrentLine++;
    return mText[mPos];
  }
}

#endif
