// runstack.hh
// string runtime stack class
// Kevin Puckett Albrecht
// Rev 4
// 1 June 1999 - 27 June 2000

#ifndef RUNSTACK_HH
#define RUNSTACK_HH

/////////////////////////
// DEFINES //////////////
/////////////////////////

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "error.hh"

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

class RunStackNode
{
  public:
    RunStackNode (char *as)
    {
      value = as;
      next = 0;
    }

    ~RunStackNode ()
    {
      // recursively delete next nodes //
      if (next) delete next;

      //BUG: unknown reason; causes crash// k_del (value);
    }

    char *value;
    RunStackNode *next;
};

class RunStack
{
  public:
    RunStack () {mHead = 0;}
    ~RunStack () {Flush ();}

    void Push (char *as);
    void PushD (char *as);
    char *Pop ();
    char *Peek ();
    void Flush ();

  private:
    RunStackNode *mHead;
};

/////////////////////////////////////
// pushes a new value to the stack //
/////////////////////////////////////
void RunStack::Push (char *as)
{
  char *sTemp = new char[strlen(as) + 1];
  k_copy (sTemp, as);
  RunStackNode *pTemp = new RunStackNode (sTemp);
  pTemp->next = mHead;
  mHead = pTemp;
}

////////////////////////////////////////////////////////
// pushes a new value to the stack, saving the memory //
////////////////////////////////////////////////////////
void RunStack::PushD (char *as)
{
  RunStackNode *pTemp = new RunStackNode (as);
  pTemp->next = mHead;
  mHead = pTemp;
}

//////////////////////////////////////////////////////////
// returns the top value on the stack after removing it //
//////////////////////////////////////////////////////////
char *RunStack::Pop ()
{
  if (mHead)
  {
    char *temp = k_clone (mHead->value);
    RunStackNode *pTemp = mHead;
    mHead = mHead->next;
    pTemp->next = NULL; // must set next to NULL so that recursive
    delete pTemp;       // ...deletion does not occur.
    return temp;
  }
  else
  {
    return NULL;
  }
}

////////////////////////////////////////////////////////////
// returns the top value on the stack without removing it //
////////////////////////////////////////////////////////////
char *RunStack::Peek ()
{
  if (mHead) return k_clone (mHead->value);
  else
  {
    return NULL;
  }
}

///////////////////////////////////
// empties the stack of all data //
///////////////////////////////////
void RunStack::Flush ()
{
  // recursively delete all nodes //
  if (mHead) delete mHead;
}

#endif