// ti.cpp
// twist interpreter
// Kevin Puckett Albrecht
// (see below for version)
// (see below for latest date)

/////////////////////////
// DEFINES //////////////
/////////////////////////

// Environment - one of the following //
#define ENV_GCC
//#define ENV_VCPP
//#define ENV_OTHER

// Operating system - one of the following //
#define OS_WINDOWS
//#define OS_BEOS

#define GLOBAL_VALUE_STR_SIZE 256
#define GLOBAL_MANTISSA_SIZE 5
#define GLOBAL_TOKEN_SIZE 256
#define GLOBAL_ERROR_STRING_SIZE 256
#define GLOBAL_FILE_LINE_SIZE 256

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <iostream.h>

// Local //
#include "k_string.hh"

#include "common.hh"
Common *gpCommon;

#include "error.hh"
Error *gpErrorP;
Error *gpErrorS;
Error *gpErrorR;

#include "inbuffer.hh"
InBuffer *gpBuffer;

#include "tree.hh"
Tree **gvpTrees;
Tree *gpTreeList;
Tree *gpTree;
TreeNode *gpStartNode; // node which holds the start label
TreeNode *gpAutoNode; // node which holds the auto variable

#include "icode.hh"
ICode *gpICode;

#include "parser.hh"

#include "exec.hh"

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

//// main ////
//////////////
int main (int nArgs, char *vsArgs[])
{
  char sFileName[256];
  k_copy (sFileName, "");
  int nMaxGotos = 100;    // set default maximum gotos to 100

  // Initialize global variables //
  gpCommon = new Common;
  gpErrorP = new Error ("Error", false);
  gpErrorS = new Error ("Error", true);
  gpErrorR = new Error ("Error", true);

  // command line argument handling //
  if (nArgs > 1)
  {
    for (int inArg = 1; inArg < nArgs; inArg++)
    {
      // '-about' //
      if ((strcmp (vsArgs[inArg], "-a") == 0) ||
          (strcmp (vsArgs[inArg], "--about") == 0)
         )
      {
        cout << "\n"
             << "============== Twist Interpreter (ti) ==============\n"
             << "\n"
             << "Author:  Kevin P. Albrecht\n"
             << "Release: 1; Build 0219; (for Windows 9x)\n"
             << "Date:    11 May 2001\n"
             << "E-mail:  kevin@albrecht.net\n"
             << "Website: http://www.eng.usf.edu/~kpalbrec/\n"
             << "\n";
        return 0;
      }
      // '-v' //
      else if ((strcmp (vsArgs[inArg], "-v") == 0) ||
               (strcmp (vsArgs[inArg], "--verbose") == 0)
              )
      {
        gpCommon->bVerbose = true;
      }
      else if ((strcmp (vsArgs[inArg], "-i") == 0) ||
               (strcmp (vsArgs[inArg], "--implicit") == 0)
              )
      {
        gpCommon->bImplicit = true;
      }
      else if ((strcmp (vsArgs[inArg], "-e") == 0) ||
               (strcmp (vsArgs[inArg], "--explicit") == 0)
              )
      {
        gpCommon->bImplicit = false;
      }
      else if (strcmp (vsArgs[inArg], "-g") == 0)
      {
        int nTemp = Value::Int (vsArgs[++inArg]);
        if (nTemp != 0) nMaxGotos = nTemp;
      }
      else
      {
        if (vsArgs[inArg][0] != '-')
        {
          k_copy (sFileName, vsArgs[inArg]);
          break;
        }
        else
        {
          gpErrorP->Show (EFF_TERMINATE, EP_INVALID_ARG, vsArgs[inArg]);
        }
      }
    }
  }
  else
  {
    cout << "Usage: ti [ option(s) ] [ filename ]\n"
         << "\n"
         << "Option           | Description                                      \n"
         << "-----------------+--------------------------------------------------\n"
         << "-a, --about      | display release and contact info                 \n"
         << "-v, --verbose    | output verbose information                       \n"
         << "-g <num>         | set maximum number of gotos to execute to <num>  \n"
         << "-i, --implicit   | activate implicit variable declarations          \n"
         << "-e, --explicit   | activate explicit variable declarations (default)\n";
    return 0;
  }

  // Finish initializing global variables //
  gpBuffer = new InBuffer (sFileName);
  gpTreeList = NULL;
  gpTree = new Tree;
  gpICode = new ICode;

  // Parse the source code //
  Error::Verbose ("MAIN", "Starting parser...", NULL, NULL, false);
  Parser *pParser = new Parser ();
  pParser->Parse();
  delete pParser;

  // Report any errors in syntax and end the program //
  if (gpCommon->nErrors > 0)
  {
    cerr << "** Syntax Errors: " << gpCommon->nErrors << "\n";
    return 0;
  }

  // Convert the symbol table for the back-end //
  gvpTrees = new Tree *[gpCommon->nTrees];
  for (Tree *pTemp = gpTreeList; pTemp; pTemp = pTemp->GetNextTree ())
  {
    pTemp->Convert (gvpTrees);
  }

  // Execute the intermediate code //
  Error::Verbose ("MAIN", "Starting executor...", NULL, NULL, false);
  Exec *pExec = new Exec (nMaxGotos);
  pExec->Execute ();
  delete pExec;
 
  return 0;
}
