// treestack.hh
// stack of trees
// Kevin Puckett Albrecht
// Rev 3
// 17 June 2000 - 27 June 2000

#ifndef TREESTACK_HH
#define TREESTACK_HH

/////////////////////////
// DEFINES //////////////
/////////////////////////

#define TREESTACK_MAX_NESTING_LEVEL 8

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <iostream.h>

// Local //
#include "common.hh"
#include "tree.hh"
#include "value.hh"

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

class TreeStack
{
  private:
    Tree *pTrees[TREESTACK_MAX_NESTING_LEVEL]; // stack

  public:
    TreeStack ();
    ~TreeStack ();

    TreeNode *Find (char *as) const;
    TreeNode *SearchAll (char *as) const;
    TreeNode *SearchLocal (char *as);

    TreeNode *EnterGlobal (char *as, Type aType = tDummy);
    TreeNode *EnterLocal (char *as, Type aType = tDummy);
    TreeNode *EnterNewLocal (char *as, Type aType = tDummy);

    Tree *GetCurrTree () const;
    void SetCurrTree (Tree *apTree);

    void EnterScope ();
    Tree *ExitScope ();
};

//// Constructor ////
TreeStack::TreeStack ()
{
  // Initialize the stack //
  gpCommon->nNestLevel = 0;
  for (int i = 1; i < TREESTACK_MAX_NESTING_LEVEL; ++i)
  {
    pTrees[i] = NULL;
  }

  // Initialize the global nesting level //
  pTrees[0] = gpTree;
}

//// Destructor ////
TreeStack::~TreeStack ()
{
}

//// Find ////
//// Returns the TreeNode; If not found, creates a new node ////
////  in the local tree.                                     ////
/////////////////////////////////////////////////////////////////
TreeNode *TreeStack::Find (char *as) const
{
  TreeNode *pNode = SearchAll (as);

  if (pNode == NULL)
  {
    gpErrorS->Show (EFF_RECOVER, ES_UNDEFINED_IDENTIFIER, as);
    pNode = pTrees[gpCommon->nNestLevel]->Enter (as);
  }

  return pNode;
}

//// SearchAll ////
//// Return a pointer to the TreeNode if found, else NULL ////
///////////////////////////////////////////////////////////////
TreeNode *TreeStack::SearchAll (char *as) const
{
  for (int i = gpCommon->nNestLevel; i >= 0; --i)
  {
    TreeNode *pNode = pTrees[i]->Search (as);
    if (pNode != NULL) return pNode;
  }

  return NULL;
}

//// SearchLocal ////
/////////////////////
TreeNode *TreeStack::SearchLocal (char *as)
{
  return pTrees[gpCommon->nNestLevel]->Search (as);
}

//// EnterGlobal ////
/////////////////////
TreeNode *TreeStack::EnterGlobal (char *as, Type aType)
{
  return pTrees[0]->Enter (as, aType);
}

//// EnterLocal ////
////////////////////
TreeNode *TreeStack::EnterLocal (char *as, Type aType)
{
  return pTrees[gpCommon->nNestLevel]->Enter (as, aType);
}

//// EnterNewLocal ////
///////////////////////
TreeNode *TreeStack::EnterNewLocal (char *as, Type aType)
{
  return pTrees[gpCommon->nNestLevel]->EnterNew (as, aType);
}

//// GetCurrTree ////
/////////////////////
Tree *TreeStack::GetCurrTree () const
{
  return pTrees[gpCommon->nNestLevel];
}

//// SetCurrTree ////
/////////////////////
void TreeStack::SetCurrTree (Tree *apTree)
{
  pTrees[gpCommon->nNestLevel] = apTree;
}

//// EnterScope ////
////////////////////
void TreeStack::EnterScope ()
{
  ++gpCommon->nNestLevel;
  //cout<<"(EnteredScope)";

  if (gpCommon->nNestLevel > TREESTACK_MAX_NESTING_LEVEL)
  {
    gpErrorS->Show (EFF_TERMINATE, ES_NESTING_TOO_DEEP);
  }

  SetCurrTree (new Tree ());
}

//// ExitScope ////
///////////////////
Tree *TreeStack::ExitScope ()
{
  //cout << "(ExitedScope)";
  return pTrees[gpCommon->nNestLevel--];
}

#endif